/*
 * Copyright 2017 Intel Corporation
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sub license, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT.
 * IN NO EVENT SHALL VMWARE AND/OR ITS SUPPLIERS BE LIABLE FOR
 * ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#include "tu_private.h"

#include "vk_util.h"

/* Convert the VK_USE_PLATFORM_* defines to booleans */
#ifdef VK_USE_PLATFORM_DISPLAY_KHR
#   undef VK_USE_PLATFORM_DISPLAY_KHR
#   define VK_USE_PLATFORM_DISPLAY_KHR true
#else
#   define VK_USE_PLATFORM_DISPLAY_KHR false
#endif
#ifdef VK_USE_PLATFORM_XLIB_KHR
#   undef VK_USE_PLATFORM_XLIB_KHR
#   define VK_USE_PLATFORM_XLIB_KHR true
#else
#   define VK_USE_PLATFORM_XLIB_KHR false
#endif
#ifdef VK_USE_PLATFORM_XLIB_XRANDR_EXT
#   undef VK_USE_PLATFORM_XLIB_XRANDR_EXT
#   define VK_USE_PLATFORM_XLIB_XRANDR_EXT true
#else
#   define VK_USE_PLATFORM_XLIB_XRANDR_EXT false
#endif
#ifdef VK_USE_PLATFORM_XCB_KHR
#   undef VK_USE_PLATFORM_XCB_KHR
#   define VK_USE_PLATFORM_XCB_KHR true
#else
#   define VK_USE_PLATFORM_XCB_KHR false
#endif
#ifdef VK_USE_PLATFORM_WAYLAND_KHR
#   undef VK_USE_PLATFORM_WAYLAND_KHR
#   define VK_USE_PLATFORM_WAYLAND_KHR true
#else
#   define VK_USE_PLATFORM_WAYLAND_KHR false
#endif
#ifdef VK_USE_PLATFORM_DIRECTFB_EXT
#   undef VK_USE_PLATFORM_DIRECTFB_EXT
#   define VK_USE_PLATFORM_DIRECTFB_EXT true
#else
#   define VK_USE_PLATFORM_DIRECTFB_EXT false
#endif
#ifdef VK_USE_PLATFORM_ANDROID_KHR
#   undef VK_USE_PLATFORM_ANDROID_KHR
#   define VK_USE_PLATFORM_ANDROID_KHR true
#else
#   define VK_USE_PLATFORM_ANDROID_KHR false
#endif
#ifdef VK_USE_PLATFORM_WIN32_KHR
#   undef VK_USE_PLATFORM_WIN32_KHR
#   define VK_USE_PLATFORM_WIN32_KHR true
#else
#   define VK_USE_PLATFORM_WIN32_KHR false
#endif
#ifdef VK_USE_PLATFORM_VI_NN
#   undef VK_USE_PLATFORM_VI_NN
#   define VK_USE_PLATFORM_VI_NN true
#else
#   define VK_USE_PLATFORM_VI_NN false
#endif
#ifdef VK_USE_PLATFORM_IOS_MVK
#   undef VK_USE_PLATFORM_IOS_MVK
#   define VK_USE_PLATFORM_IOS_MVK true
#else
#   define VK_USE_PLATFORM_IOS_MVK false
#endif
#ifdef VK_USE_PLATFORM_MACOS_MVK
#   undef VK_USE_PLATFORM_MACOS_MVK
#   define VK_USE_PLATFORM_MACOS_MVK true
#else
#   define VK_USE_PLATFORM_MACOS_MVK false
#endif
#ifdef VK_USE_PLATFORM_METAL_EXT
#   undef VK_USE_PLATFORM_METAL_EXT
#   define VK_USE_PLATFORM_METAL_EXT true
#else
#   define VK_USE_PLATFORM_METAL_EXT false
#endif
#ifdef VK_USE_PLATFORM_FUCHSIA
#   undef VK_USE_PLATFORM_FUCHSIA
#   define VK_USE_PLATFORM_FUCHSIA true
#else
#   define VK_USE_PLATFORM_FUCHSIA false
#endif
#ifdef VK_USE_PLATFORM_GGP
#   undef VK_USE_PLATFORM_GGP
#   define VK_USE_PLATFORM_GGP true
#else
#   define VK_USE_PLATFORM_GGP false
#endif
#ifdef VK_ENABLE_BETA_EXTENSIONS
#   undef VK_ENABLE_BETA_EXTENSIONS
#   define VK_ENABLE_BETA_EXTENSIONS true
#else
#   define VK_ENABLE_BETA_EXTENSIONS false
#endif

/* And ANDROID too */
#ifdef ANDROID
#   undef ANDROID
#   define ANDROID true
#else
#   define ANDROID false
#   define ANDROID_API_LEVEL 0
#endif

#define TU_HAS_SURFACE (VK_USE_PLATFORM_WAYLAND_KHR ||                                        VK_USE_PLATFORM_XCB_KHR ||                                        VK_USE_PLATFORM_XLIB_KHR ||                                        VK_USE_PLATFORM_DISPLAY_KHR)

static const uint32_t MAX_API_VERSION = VK_MAKE_VERSION(1, 2, 131);

VkResult tu_EnumerateInstanceVersion(
    uint32_t*                                   pApiVersion)
{
    *pApiVersion = MAX_API_VERSION;
    return VK_SUCCESS;
}

const VkExtensionProperties tu_instance_extensions[TU_INSTANCE_EXTENSION_COUNT] = {
   {"VK_KHR_get_display_properties2", 1},
   {"VK_KHR_get_physical_device_properties2", 1},
   {"VK_KHR_get_surface_capabilities2", 1},
   {"VK_KHR_surface", 25},
   {"VK_KHR_wayland_surface", 6},
   {"VK_KHR_xcb_surface", 6},
   {"VK_KHR_xlib_surface", 6},
   {"VK_KHR_display", 23},
   {"VK_EXT_direct_mode_display", 1},
   {"VK_EXT_acquire_xlib_display", 1},
   {"VK_EXT_display_surface_counter", 1},
   {"VK_EXT_debug_report", 9},
   {"VK_KHR_external_memory_capabilities", 1},
   {"VK_KHR_external_semaphore_capabilities", 1},
};

const struct tu_instance_extension_table tu_instance_extensions_supported = {
   .KHR_get_display_properties2 = (!ANDROID || ANDROID_API_LEVEL >= 29) && (VK_USE_PLATFORM_DISPLAY_KHR),
   .KHR_get_physical_device_properties2 = (!ANDROID || ANDROID_API_LEVEL >= 26) && (true),
   .KHR_get_surface_capabilities2 = (!ANDROID || ANDROID_API_LEVEL >= 26) && (TU_HAS_SURFACE),
   .KHR_surface = (!ANDROID || ANDROID_API_LEVEL >= 26) && (TU_HAS_SURFACE),
   .KHR_wayland_surface = (!ANDROID || ANDROID_API_LEVEL >= 26) && (VK_USE_PLATFORM_WAYLAND_KHR),
   .KHR_xcb_surface = (!ANDROID || ANDROID_API_LEVEL >= 26) && (VK_USE_PLATFORM_XCB_KHR),
   .KHR_xlib_surface = (!ANDROID || ANDROID_API_LEVEL >= 26) && (VK_USE_PLATFORM_XLIB_KHR),
   .KHR_display = (!ANDROID || ANDROID_API_LEVEL >= 26) && (VK_USE_PLATFORM_DISPLAY_KHR),
   .EXT_direct_mode_display = VK_USE_PLATFORM_DISPLAY_KHR,
   .EXT_acquire_xlib_display = VK_USE_PLATFORM_XLIB_XRANDR_EXT,
   .EXT_display_surface_counter = VK_USE_PLATFORM_DISPLAY_KHR,
   .EXT_debug_report = true,
   .KHR_external_memory_capabilities = (!ANDROID || ANDROID_API_LEVEL >= 28) && (true),
   .KHR_external_semaphore_capabilities = (!ANDROID || ANDROID_API_LEVEL >= 28) && (true),
};

uint32_t
tu_physical_device_api_version(struct tu_physical_device *device)
{
    uint32_t version = 0;

    uint32_t override = vk_get_version_override();
    if (override)
        return MIN2(override, MAX_API_VERSION);

    if (!(true))
        return version;
    version = VK_MAKE_VERSION(1, 2, 131);

    return version;
}

const VkExtensionProperties tu_device_extensions[TU_DEVICE_EXTENSION_COUNT] = {
   {"VK_KHR_bind_memory2", 1},
   {"VK_KHR_create_renderpass2", 1},
   {"VK_KHR_dedicated_allocation", 1},
   {"VK_KHR_get_memory_requirements2", 1},
   {"VK_KHR_maintenance1", 1},
   {"VK_KHR_maintenance2", 1},
   {"VK_KHR_maintenance3", 1},
   {"VK_KHR_sampler_mirror_clamp_to_edge", 1},
   {"VK_KHR_sampler_ycbcr_conversion", 1},
   {"VK_KHR_swapchain", 68},
   {"VK_EXT_display_control", 1},
   {"VK_KHR_external_memory", 1},
   {"VK_KHR_external_memory_fd", 1},
   {"VK_EXT_external_memory_dma_buf", 1},
   {"VK_EXT_image_drm_format_modifier", 1},
   {"VK_EXT_sample_locations", 1},
   {"VK_EXT_sampler_filter_minmax", 1},
   {"VK_EXT_transform_feedback", 1},
   {"VK_ANDROID_native_buffer", 1},
   {"VK_KHR_external_fence", 1},
   {"VK_KHR_external_fence_fd", 1},
   {"VK_KHR_external_semaphore", 1},
   {"VK_KHR_external_semaphore_fd", 1},
   {"VK_IMG_filter_cubic", 1},
   {"VK_EXT_filter_cubic", 1},
   {"VK_EXT_index_type_uint8", 1},
   {"VK_EXT_vertex_attribute_divisor", 1},
   {"VK_KHR_shader_draw_parameters", 1},
   {"VK_KHR_variable_pointers", 1},
   {"VK_EXT_private_data", 1},
   {"VK_EXT_shader_stencil_export", 1},
   {"VK_EXT_depth_clip_enable", 1},
   {"VK_KHR_draw_indirect_count", 1},
   {"VK_EXT_4444_formats", 1},
   {"VK_EXT_conditional_rendering", 1},
   {"VK_EXT_custom_border_color", 12},
   {"VK_KHR_multiview", 1},
   {"VK_EXT_host_query_reset", 1},
   {"VK_EXT_shader_viewport_index_layer", 1},
   {"VK_EXT_extended_dynamic_state", 1},
   {"VK_KHR_push_descriptor", 1},
   {"VK_KHR_incremental_present", 1},
   {"VK_KHR_image_format_list", 1},
   {"VK_KHR_depth_stencil_resolve", 1},
   {"VK_KHR_performance_query", 1},
};

void
tu_physical_device_get_supported_extensions(const struct tu_physical_device *device,
                                                   struct tu_device_extension_table *extensions)
{
   *extensions = (struct tu_device_extension_table) {
      .KHR_bind_memory2 = (!ANDROID || ANDROID_API_LEVEL >= 28) && (true),
      .KHR_create_renderpass2 = (!ANDROID || ANDROID_API_LEVEL >= 28) && (true),
      .KHR_dedicated_allocation = (!ANDROID || ANDROID_API_LEVEL >= 28) && (true),
      .KHR_get_memory_requirements2 = (!ANDROID || ANDROID_API_LEVEL >= 28) && (true),
      .KHR_maintenance1 = (!ANDROID || ANDROID_API_LEVEL >= 26) && (true),
      .KHR_maintenance2 = (!ANDROID || ANDROID_API_LEVEL >= 28) && (true),
      .KHR_maintenance3 = (!ANDROID || ANDROID_API_LEVEL >= 28) && (true),
      .KHR_sampler_mirror_clamp_to_edge = (!ANDROID || ANDROID_API_LEVEL >= 26) && (true),
      .KHR_sampler_ycbcr_conversion = (!ANDROID || ANDROID_API_LEVEL >= 28) && (true),
      .KHR_swapchain = (!ANDROID || ANDROID_API_LEVEL >= 26) && (TU_HAS_SURFACE),
      .EXT_display_control = VK_USE_PLATFORM_DISPLAY_KHR,
      .KHR_external_memory = (!ANDROID || ANDROID_API_LEVEL >= 28) && (true),
      .KHR_external_memory_fd = (!ANDROID || ANDROID_API_LEVEL >= 28) && (true),
      .EXT_external_memory_dma_buf = true,
      .EXT_image_drm_format_modifier = true,
      .EXT_sample_locations = device->gpu_id == 650,
      .EXT_sampler_filter_minmax = true,
      .EXT_transform_feedback = true,
      .ANDROID_native_buffer = (!ANDROID || ANDROID_API_LEVEL >= 26) && (ANDROID),
      .KHR_external_fence = (!ANDROID || ANDROID_API_LEVEL >= 28) && (true),
      .KHR_external_fence_fd = (!ANDROID || ANDROID_API_LEVEL >= 28) && (true),
      .KHR_external_semaphore = (!ANDROID || ANDROID_API_LEVEL >= 28) && (true),
      .KHR_external_semaphore_fd = (!ANDROID || ANDROID_API_LEVEL >= 28) && (true),
      .IMG_filter_cubic = device->gpu_id == 650,
      .EXT_filter_cubic = device->gpu_id == 650,
      .EXT_index_type_uint8 = true,
      .EXT_vertex_attribute_divisor = true,
      .KHR_shader_draw_parameters = (!ANDROID || ANDROID_API_LEVEL >= 26) && (true),
      .KHR_variable_pointers = (!ANDROID || ANDROID_API_LEVEL >= 28) && (true),
      .EXT_private_data = true,
      .EXT_shader_stencil_export = true,
      .EXT_depth_clip_enable = true,
      .KHR_draw_indirect_count = (!ANDROID || ANDROID_API_LEVEL >= 28) && (true),
      .EXT_4444_formats = true,
      .EXT_conditional_rendering = true,
      .EXT_custom_border_color = true,
      .KHR_multiview = (!ANDROID || ANDROID_API_LEVEL >= 28) && (true),
      .EXT_host_query_reset = true,
      .EXT_shader_viewport_index_layer = true,
      .EXT_extended_dynamic_state = true,
      .KHR_push_descriptor = (!ANDROID || ANDROID_API_LEVEL >= 26) && (true),
      .KHR_incremental_present = (!ANDROID || ANDROID_API_LEVEL >= 26) && (TU_HAS_SURFACE),
      .KHR_image_format_list = (!ANDROID || ANDROID_API_LEVEL >= 28) && (true),
      .KHR_depth_stencil_resolve = (!ANDROID || ANDROID_API_LEVEL >= 29) && (true),
      .KHR_performance_query = (!ANDROID || ANDROID_API_LEVEL >= 30) && (device->instance->debug_flags & TU_DEBUG_PERFC),
   };
}
